﻿# Do not remove this test for UTF-8: if “Ω” doesn’t appear as greek uppercase omega letter enclosed in quotation marks, you should use an editor that supports UTF-8, not this one.

$ErrorActionPreference = 'Stop';

$version     = "1.51.0"
$packageName = $env:chocolateyPackageName
$toolsDir    = "$(Split-Path -parent $MyInvocation.MyCommand.Definition)"

$rustcUrl = "https://static.rust-lang.org/dist/2021-03-25/rustc-1.51.0-i686-pc-windows-gnu.tar.gz"
$rustcUrl64 = "https://static.rust-lang.org/dist/2021-03-25/rustc-1.51.0-x86_64-pc-windows-gnu.tar.gz"

$cargoUrl = "https://static.rust-lang.org/dist/2021-03-25/cargo-1.51.0-i686-pc-windows-gnu.tar.gz"
$cargoUrl64 = "https://static.rust-lang.org/dist/2021-03-25/cargo-1.51.0-x86_64-pc-windows-gnu.tar.gz"

$stdUrl = "https://static.rust-lang.org/dist/2021-03-25/rust-std-1.51.0-i686-pc-windows-gnu.tar.gz"
$stdUrl64 = "https://static.rust-lang.org/dist/2021-03-25/rust-std-1.51.0-x86_64-pc-windows-gnu.tar.gz"

$packageArgs = @{
    packageName    = $packageName
    unzipLocation  = $toolsDir
    url            = $rustcUrl
    checksum       = "23e48ecf4177e55e83c95531bf224e6c2af914159655886f82a282934468f81a"
    checksumType   = "sha256"
    url64bit       = $rustcUrl64
    checksum64     = "0a8a0074ec615ab54f290da123cdeeee0e3cfa82421b783e81e5cc20d60fc2d1"
    checksumType64 = "sha256"
}

$packageSrcArgs = @{
    packageName    = $packageName
    unzipLocation  = $toolsDir
    url            = "https://static.rust-lang.org/dist/2021-03-25/rust-src-1.51.0.tar.gz"
    checksum       = "18904ca04c5bc09fa1a88d32391c611bc60bc3a739551496cad0829e34301563"
    checksumType   = "sha256"
}

$packageCargoArgs = @{
    packageName    = $packageName
    unzipLocation  = $toolsDir
    url            = $cargoUrl
    checksum       = "3e54286c6a4b6c05834ecae2e46a54853b0ac5a6d149e8c72bdd210cc6afda52"
    checksumType   = "sha256"
    url64bit       = $cargoUrl64
    checksum64     = "d1c3abe4cf3bb0d52e9946b0ff823a85b3931562424d2a5377b77bbf30deaccc"
    checksumType64 = "sha256"
}

$packageStdArgs = @{
    packageName    = $packageName
    unzipLocation  = $toolsDir
    url            = $stdUrl
    checksum       = "a69fc6dfc61e7cc6004cb1fe4a0f71e160d4a023bbca4725293dd7a78be15692"
    checksumType   = "sha256"
    url64bit       = $stdUrl64
    checksum64     = "55f871bdaf361a26280ca5396297cc7c67237cd86d4ebfe3cbdf9fac14ce0327"
    checksumType64 = "sha256"
}

$packageMingwArgs = @{
    packageName    = $packageName
    unzipLocation  = $toolsDir
    url            = "https://static.rust-lang.org/dist/2021-03-25/rust-mingw-1.51.0-i686-pc-windows-gnu.tar.gz"
    checksum       = "a9b2bc6c6301dc877ee8eb87045ac04a2ab36b9344234e1244c936328be4a6c3"
    checksumType   = "sha256"
    url64bit       = "https://static.rust-lang.org/dist/2021-03-25/rust-mingw-1.51.0-x86_64-pc-windows-gnu.tar.gz"
    checksum64     = "9674ab856018e3a8b8718dbc76c62c10d33899a2e66c20ab1d88327a62286e6e"
    checksumType64 = "sha256"
}

# Updates require us to get rid of the existing installation
# https://chocolatey.org/packages/rust#comment-4632965834
if (Test-Path $toolsDir\bin) { rm -Recurse -Force $toolsDir\bin }
if (Test-Path $toolsDir\etc) { rm -Recurse -Force $toolsDir\etc }
if (Test-Path $toolsDir\lib) { rm -Recurse -Force $toolsDir\lib }
if (Test-Path $toolsDir\share) { rm -Recurse -Force $toolsDir\share }

# Note to the reader: Install-ChocolateyZipFile only extracts one layer,
# so it turns the tar.gz files that Rust distributes into bar tar files.
# Useless.
Install-ChocolateyZipPackage @packageArgs
Get-ChocolateyUnzip -FileFullPath $toolsDir/rustc-$version-i686-pc-windows-gnu.tar -FileFullPath64 $toolsDir/rustc-$version-x86_64-pc-windows-gnu.tar -Destination $toolsDir
Install-ChocolateyZipPackage @packageSrcArgs
Get-ChocolateyUnzip -FileFullPath $toolsDir/rust-src-$version.tar -Destination $toolsDir
Install-ChocolateyZipPackage @packageCargoArgs
Get-ChocolateyUnzip -FileFullPath $toolsDir/cargo-$version-i686-pc-windows-gnu.tar -FileFullPath64 $toolsDir/cargo-$version-x86_64-pc-windows-gnu.tar -Destination $toolsDir
Install-ChocolateyZipPackage @packageStdArgs
Get-ChocolateyUnzip -FileFullPath $toolsDir/rust-std-$version-i686-pc-windows-gnu.tar -FileFullPath64 $toolsDir/rust-std-$version-x86_64-pc-windows-gnu.tar -Destination $toolsDir
# This is basically what install.sh does, though with less customizability,
# because we delegate to Chocolatey for things like uninstalling and deciding where $toolsDir is.
function Install-RustPackage([string]$Directory) {
  cd $Directory
  cat components | foreach {
    $c = $_
    cat $Directory/$c/manifest.in | foreach {
      # https://community.chocolatey.org/packages/rust#comment-5339041282
      if (!$_.Contains("bash_completion.d")) {
        if ($_.StartsWith("file:")) {
          $f = $_.SubString(5)
          $d = (split-path -parent $f)
          if (!(test-path $toolsDir/$d)) { mkdir $toolsDir/$d }
          mv $Directory/$c/$f $toolsDir/$f
        }
        # The assumption is that a manifest with a `dir:` directive is the sole provider of that directory,
        # unlike other rust components, where we're expected to merge the directories together.
        # Only component I've found with a `dir:` directive, currently, is rust-docs.
        if ($_.StartsWith("dir:")) {
          $f = $_.SubString(4)
          $d = (split-path -parent $f)
          if (!(test-path $toolsDir/$d)) { mkdir $toolsDir/$d }
          mv $Directory/$c/$f $toolsDir/$f
        }
      }
    }
  }
  cd $toolsDir
}
rm -recurse -force $toolsDir/rustc-$version-*.tar
rm -recurse -force $toolsDir/rust-src-$version.tar
rm -recurse -force $toolsDir/rust-std-$version-*.tar
rm -recurse -force $toolsDir/cargo-$version-*.tar
dir $toolsDir/rustc-$version-* | foreach { Install-RustPackage (join-path $_ '') }
dir $toolsDir/cargo-$version-* | foreach { Install-RustPackage (join-path $_ '') }
dir $toolsDir/rust-std-$version-* | foreach { Install-RustPackage (join-path $_ '') }
Install-RustPackage $toolsDir/rust-src-$version
rm -recurse -force $toolsDir/rustc-$version-*
rm -recurse -force $toolsDir/cargo-$version-*
rm -recurse -force $toolsDir/rust-std-$version-*
rm -recurse -force $toolsDir/rust-src-$version
if ("https://static.rust-lang.org/dist/2021-03-25/rust-mingw-1.51.0-i686-pc-windows-gnu.tar.gz" -ne "") {
  Install-ChocolateyZipPackage @packageMingwArgs
  Get-ChocolateyUnzip -FileFullPath $toolsDir/rust-mingw-$version-i686-pc-windows-gnu.tar -FileFullPath64 $toolsDir/rust-mingw-$version-x86_64-pc-windows-gnu.tar -Destination $toolsDir
  rm -recurse -force $toolsDir/rust-mingw-$version-*.tar
  dir $toolsDir/rust-mingw-$version-* | foreach { Install-RustPackage (join-path $_ '') }
  rm -recurse -force $toolsDir/rust-mingw-$version-*
}
# Mark gcc.exe, and its relatives, as not-for-shimming.
# https://chocolatey.org/packages/rust#comment-4690124900
$files = Get-ChildItem $toolsDir\lib\rustlib\ -include '*.exe' -recurse -name
foreach ($file in $files) {
  New-Item "$toolsDir\lib\rustlib\$file.ignore" -type file -force | Out-Null
}
